#pragma once
#include <vector>
#include <utility>
#include <string>

struct Pair {
  size_t j = 0;
  size_t i = 0;
};

typedef std::vector<Pair> PairList;
typedef std::vector<int> MatrixRow;
typedef std::vector<MatrixRow> Matrix;

void dump_map(const std::string &msg, Matrix &map);
void add_neighbors(Matrix &closed, size_t j, size_t i);

class Map {
  Matrix m_input_map;
  Matrix m_walls;
  Matrix m_paths;
  int m_limit = 0;
public:

  void make_paths();

  Matrix& paths() { return m_paths; }
  Matrix& input_map() { return m_input_map; }
  Matrix& walls() { return m_walls; }
  int limit() { return m_limit; }
  size_t width() { return m_walls[0].size(); }
  size_t height() { return m_walls.size(); }

  void make_room(size_t origin_y, size_t origin_x, size_t width, size_t height);

  void dump() {
    dump_map("PATHS", m_paths);
    dump_map("WALLS", m_walls);
    dump_map("INPUT", m_input_map);
  }

  void generate();

  Map(Matrix input_map, Matrix walls_map, int limit) :
    m_input_map(input_map),
    m_walls(walls_map), m_limit(limit) {
    }

  Map(size_t width, size_t height);

  Map(Map &map) = delete;
};
